package models;

import java.util.Random;


/** 
 * implements a Random terrain generator using a 2 dimensional boolean array
 * <ul>
 * <li> <tt> true </tt> - a wall is present </li>
 * <li> <tt> false </tt> - a space is present </li>
 * </ul>
 * 
 * Valid array positions are: <tt> [column][row]  in [0..width-1][0..height-1] </tt>
 * 
 * @author J Paul Gibson
 * @version 1
 */
public class RandomTerrain {

	/**
	 * the positions in the grid are either:
	 * <ul>
     * <li> <tt> true </tt> - a wall is present </li>
     * <li> <tt> false </tt> - a space is present </li>
	 * </ul>
	 */
	private boolean [][] position;
	
	/**
	 * The width of the grid - must be a positive integer
	 */
	protected final int width;
	
	/**
	 * The height of the grid - must be a positive integer
	 */
	protected final int height;
	
	/**
	 * We implement a grid as a 2 dimensional array of boolean values, originally initiailises all elements
	 *  to false to indicate no walls 
	 * @param numberOfColumns is the grid width
	 * @param numberOfRows is the grid height
	 */
	public RandomTerrain (int numberOfColumns, int numberOfRows){
		
		width = numberOfColumns; 
		height = numberOfRows;
		position = new boolean[width][height];
		removeAllWalls();
	}
	
	
   public static String PrintRandomTerrain (Random RNG){
	   
	   
	   RandomTerrain terrain;
		
		int width = 10+ RNG.nextInt(41); // random value between 10 and 40
		
		System.out.println("width= "+width);
		
		int height = 10+ RNG.nextInt(41); // random value between 10 and 40
		
		System.out.println("height= "+height);
		
		terrain = new RandomTerrain(width, height);
		
		double chanceOfWall = RNG.nextDouble();
		
		System.out.println("chanceOfWall= "+chanceOfWall);
	
		terrain.removeAllWalls();
		terrain.randomFill(chanceOfWall, RNG);
		
		System.out.println(terrain);
		
		return terrain.toString();
	}
	
	
	/**
	 * 
	 * @param chanceOfWall must be a value between 0.0 and 1.0 and 
	 *       corresponds to the chance of a wall occurring in a location in the grid
	 * @param RNG the random number generator used in the random constructor
	 */
	public void randomFill(double chanceOfWall, Random RNG){
		
		removeAllWalls();
		for (int row = 0; row< height; row++) 
			for (int col = 0; col< width; col++) 
				 if (RNG.nextFloat()<=chanceOfWall) position[col][row]=true;
		
	}
	

	public void removeAllWalls(){
		for (int row = 0; row< height; row++) 
			for (int col = 0; col< width; col++) 
				position[col][row]= false;
	};

	public void fillWithWalls(){
		for (int row = 0; row< height; row++) 
			for (int col = 0; col< width; col++) 
				position[col][row]= true;
	};
	

	public boolean isWall (int horizontalPos, int verticalPos) throws IllegalArgumentException{
		
		if (horizontalPos<0 || horizontalPos>=width || verticalPos<0 || verticalPos >= height)
			throw new IllegalArgumentException("The position specified is not in the grid");
		return position[horizontalPos][verticalPos];
	}
	

	public boolean isSpace (int horizontalPos, int verticalPos) throws IllegalArgumentException{
		return !isWall(horizontalPos, verticalPos);
	}
	

	public int getWidth(){return width;}
	

	public int getHeight(){return height;}
	

	public void addWall(int horizontalPos, int verticalPos) throws IllegalArgumentException, IllegalStateException{
		if (horizontalPos<0 || horizontalPos>=width || verticalPos<0 || verticalPos >= height)
			throw new IllegalArgumentException("The position specified is not in the grid");
		else if (position[horizontalPos][verticalPos])
			throw new IllegalStateException("The position is already a wall");
		else position[horizontalPos][verticalPos]=true;
		
	}
	

	public void removeWall(int horizontalPos, int verticalPos) throws IllegalArgumentException, IllegalStateException{
		if (horizontalPos<0 || horizontalPos>=width || verticalPos<0 || verticalPos >= height)
			throw new IllegalArgumentException("The position specified is not in the grid");
		else if (!position[horizontalPos][verticalPos])
			throw new IllegalStateException("The position is already a space");
		else position[horizontalPos][verticalPos]=false;
	}
	

	public boolean invariant(){
		return (width>0 && height >0);
	}
	
	/**
	 *The grid of boolean values as a string of characters, where true is represented by a '#' and false by ' '<br>
	 *The origin of the grid is the first character generated 
	 *and so will be at the top left when printed to the output stream<br>
	 *The grid string is generated row by row with a newline between each row
	 */
	public String toString(){
		String str = "";
		for (int row = 0; row< height; row++) 
			{str=str+"\n";
			 for (int col = 0; col< width; col++)
				  if (position[col][row]) str = str+'#';
				      else str = str+' ';
			}
		return str;	
	}
	

	public boolean equals(Object obj) {
		if (obj == null) return false;
		if (this == obj) return true;
		if(!(obj instanceof RandomTerrain)) return false;
		RandomTerrain g = (RandomTerrain) obj;
		if (height!= g.getHeight()) return false;
		if (width!= g.getWidth()) return false;
		for (int row = 0; row< height; row++) 
			for (int col = 0; col< width; col++) 
				if (isWall(col,row) != g.isWall(col,row)) return false;	 
		return true;
	}

}
